//:ts=8  -*- C++ -*-
/*****************************************************************************
 *
 *   |_|_|_  |_|_    |_    |_|_|_  |_		     C O M M U N I C A T I O N
 * |_        |_  |_  |_  |_        |_		               N E T W O R K S
 * |_        |_  |_  |_  |_        |_		                     C L A S S
 *   |_|_|_  |_    |_|_    |_|_|_  |_|_|_|_	                 L I B R A R Y
 *
 * $Id: HyperGeom.h,v 0.27 1995/01/20 15:13:51 cncl-adm Exp cncl-adm $
 *
 * CNClass: CNHyperGeom --- Hyper geometric distributed random numbers
 *
 *****************************************************************************
 * Copyright (C) 1992-1995   Communication Networks
 *                           Aachen University of Technology
 *                           D-52056 Aachen
 *                           Germany
 *                           Email: cncl-adm@dfv.rwth-aachen.de
 *****************************************************************************
 * This file is part of the CN class library. All files marked with
 * this header are free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.  This library is
 * distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
 * License for more details.  You should have received a copy of the GNU
 * Library General Public License along with this library; if not, write
 * to the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *****************************************************************************
 * original Copyright:
 * -------------------
 * Copyright (C) 1988 Free Software Foundation
 *    written by Dirk Grunwald (grunwald@cs.uiuc.edu)
 * 
 * This file is part of the GNU C++ Library.  This library is free
 * software; you can redistribute it and/or modify it under the terms of
 * the GNU Library General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your
 * option) any later version.  This library is distributed in the hope
 * that it will be useful, but WITHOUT ANY WARRANTY; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU Library General Public License for more details.
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************/

#ifndef __HyperGeom_h
#define __HyperGeom_h


#include <CNCL/Class.h>
#include <CNCL/Param.h>

#include <CNCL/Random.h>		// Base class

extern CNClassDesc CN_HYPERGEOM;	// CNClass CNHyperGeom description



/*
 * The class CNHyperGeom
 */

class CNHyperGeom : public CNRandom
{

protected:
    double pMean;
    double pVariance;
    double pP;
    void setState();

public:

    double mean();
    double mean(double x);
    double variance();
    double variance(double x);

    virtual double operator()();
    
    /***** Constructors ******************************************************/
public:
    CNHyperGeom();   				// Default constructor
    CNHyperGeom(CNParam *param);			// CNParam constructor
    CNHyperGeom(double mean, double variance, CNRNG *gen);

    /***** Member functions required by CNCL *********************************/
public:
    virtual CNClassDesc class_desc() const	// CNClass description
    {
	return CN_HYPERGEOM;
    };
            
    virtual bool is_a(CNClassDesc desc) const	// Type checking
    {
	return desc == CN_HYPERGEOM ? TRUE : CNRandom::is_a(desc);
    };
        
    static CNHyperGeom *cast_from_object(CNObject *obj) // Safe type cast
    {
#   ifdef NO_TYPE_CHECK
	return (CNHyperGeom *)obj;
#   else
	return (CNHyperGeom *)( !obj || obj->is_a(CN_HYPERGEOM)
	       ? obj : fatal_type(obj->class_desc(), CN_HYPERGEOM) );
#   endif
    }
    
    static CNObject *new_object(CNParam *param = NIL) // Object creation
    { return param ? new CNHyperGeom(param) : new CNHyperGeom; }
    
    // Print/debug output
    virtual void print(ostream &strm = cout) const;
    virtual void dump (ostream &strm = cout) const;
};



inline void CNHyperGeom::setState()
{
    double z = pVariance / (pMean * pMean);
    pP = 0.5 * (1.0 - sqrt((z - 1.0) / ( z + 1.0 )));
}

inline CNHyperGeom::CNHyperGeom(double mean, double variance, CNRNG *gen)
    : CNRandom(gen)
{
    pMean = mean; 
    pVariance = variance;
    setState();
}

inline CNHyperGeom::CNHyperGeom()
	 : CNRandom((CNRNG*)NIL)
{
    pMean = 0;
    pVariance = 0;
    pP = 0;
}

inline CNHyperGeom::CNHyperGeom(CNParam *param)
	 : CNRandom((CNRNG*)NIL)
{
    pMean = 0;
    pVariance = 0;
    pP = 0;
}
	
inline double CNHyperGeom::mean() { return pMean; };

inline double CNHyperGeom::mean(double x)
{
    double t = pMean; 
    pMean = x;
    setState(); 
    return t;
}

inline double CNHyperGeom::variance() { return pVariance; }

inline double CNHyperGeom::variance(double x)
{
    double t = pVariance; 
    pVariance = x;
    setState(); 
    return t;
}

#endif /**__HyperGeom_h**/


